import { Constants } from "./Constants.js";
import { PingCompensatedCharacter } from "./PingCompensatedCharacter.js";
export class Ranger extends PingCompensatedCharacter {
    ctype = "ranger";
    async fiveShot(target1, target2, target3, target4, target5) {
        if (!this.ready)
            throw new Error("We aren't ready yet [fiveShot].");
        const projectiles = [];
        const getProjectiles = (data) => {
            if (data.attacker == this.id
                && data.type == "5shot"
                && (data.target == target1 || data.target == target2 || data.target == target3 || data.target == target4 || data.target == target5)) {
                projectiles.push(data.pid);
            }
        };
        this.socket.on("action", getProjectiles);
        try {
            const response = this.getResponsePromise("5shot");
            this.socket.emit("skill", {
                ids: [target1, target2, target3, target4, target5],
                name: "5shot"
            });
            await response;
        }
        finally {
            this.socket.off("action", getProjectiles);
        }
        return projectiles;
    }
    // NOTE: UNTESTED
    async fourFinger(target) {
        if (!this.ready)
            throw new Error("We aren't ready yet [fourFinger].");
        // TODO: Check that the target is not a monster.
        const marked = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]4fingers['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.off("eval", cooldownCheck);
                reject(`fourfinger timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "4fingers"
        });
        return marked;
    }
    async huntersMark(target) {
        if (!this.ready)
            throw new Error("We aren't ready yet [huntersMark].");
        const marked = new Promise((resolve, reject) => {
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]huntersmark['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("eval", cooldownCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.off("eval", cooldownCheck);
                reject(`huntersmark timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", {
            id: target,
            name: "huntersmark"
        });
        return marked;
    }
    async piercingShot(target) {
        if (!this.ready)
            throw new Error("We aren't ready yet [piercingShot].");
        if (this.G.skills.piercingshot.mp > this.mp)
            throw new Error("Not enough MP to use piercingShot");
        const piercingShotStarted = new Promise((resolve, reject) => {
            let projectile;
            const attackCheck = (data) => {
                if (data.attacker == this.id
                    && data.type == "piercingshot"
                    && data.target == target) {
                    projectile = data.pid;
                }
            };
            const cooldownCheck = (data) => {
                // NOTE: This is `attack`, not `piercingshot`, because they share a cooldown.
                if (/skill_timeout\s*\(\s*['"]attack['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectile);
                }
            };
            setTimeout(() => {
                this.socket.off("action", attackCheck);
                this.socket.off("eval", cooldownCheck);
                reject(`piercingshot timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", { id: target, name: "piercingshot" });
        return piercingShotStarted;
    }
    // NOTE: UNTESTED
    async poisonArrow(target, poison = this.locateItem("poison")) {
        if (!this.ready)
            throw new Error("We aren't ready yet [poisonArrow].");
        if (poison === undefined)
            throw new Error("We need poison to use this skill.");
        const poisonArrowed = new Promise((resolve, reject) => {
            let projectile;
            const attackCheck = (data) => {
                if (data.attacker == this.id
                    && data.type == "poisonarrow"
                    && data.target == target) {
                    projectile = data.pid;
                }
            };
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]poisonarrow['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.off("action", attackCheck);
                    this.socket.off("eval", cooldownCheck);
                    resolve(projectile);
                }
            };
            setTimeout(() => {
                this.socket.off("action", attackCheck);
                this.socket.off("eval", cooldownCheck);
                reject(`poisonarrow timeout (${Constants.TIMEOUT}ms)`);
            }, Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", { id: target, name: "poisonarrow", num: poison });
        return poisonArrowed;
    }
    /**
     * @param {string} target
     * @return {*}  {Promise<string>}
     * @memberof Ranger
     */
    async superShot(target) {
        if (!this.ready)
            throw new Error("We aren't ready yet [superShot].");
        // Set up to return the projectile for the super shot.
        let projectile;
        const getProjectile = (data) => {
            if (data.attacker == this.id
                && data.type == "supershot"
                && data.target == target) {
                projectile = data.pid;
            }
        };
        this.socket.on("action", getProjectile);
        try {
            const response = this.getResponsePromise("supershot");
            this.socket.emit("skill", { id: target, name: "supershot" });
            await response;
        }
        finally {
            this.socket.off("action", getProjectile);
        }
        return projectile;
    }
    async threeShot(target1, target2, target3) {
        if (!this.ready)
            throw new Error("We aren't ready yet [threeShot].");
        const projectiles = [];
        const getProjectiles = (data) => {
            if (data.attacker == this.id
                && data.type == "3shot"
                && (data.target == target1 || data.target == target2 || data.target == target3)) {
                projectiles.push(data.pid);
            }
        };
        this.socket.on("action", getProjectiles);
        try {
            const response = this.getResponsePromise("3shot");
            this.socket.emit("skill", {
                ids: [target1, target2, target3],
                name: "3shot"
            });
            await response;
        }
        finally {
            this.socket.off("action", getProjectiles);
        }
        return projectiles;
    }
}
//# sourceMappingURL=Ranger.js.map